/************************************************************

Code based on:
Copyright (C) 1998, Lucent Technologies
with serious rewriting by:
CopyLeft (L) 2005, Peter A. Cejchan (partim)
All rights reserved

************************************************************/

#include "plot.h"
#include <u.h>
#include <libc.h>

/* Plotting functions for PostScript */

static double _a,_b,_c,_d, _ox, _oy;
static double width = 0;

#define xs(_x) (_ox = _a*(_x)+_b)
#define ys(_y) (_oy = _c*(_y)+_d)

void
range(int xmin, int ymin, int xmax, int ymax)
{
	_a = 6.5*72/(xmax-xmin);
	_b = -_a*xmin + 1.0*72;
	_c = 6.5*72/(ymax-ymin);
	_d = -_c*ymin + 3.5*72;
}

void
scale(int invscale)
{
	_a = 40*72;
	_b = -_a + 0.5*72;
	_c = 40*72;
	_d = -_a + 0.5*72;
}

/* open the plotting output */
void
openpl(void)
{
	print("%%!PS\n"
		"%%Creator: histog\n"
		"%%BoundingBox: 72 252 540 720\n"
		"%%EndComments\n"
		"/co /sethsbcolor load def\n"
		"/m /moveto load def\n"
		"/c /curveto load def\n"
		"/cp /closepath load def\n"
		"/np /closepath load def\n"
		"/s /stroke load def\n"
		"/f /fill load def\n"
		"/t {gsave newpath 0 0 moveto (X) true charpath "
		"flattenpath pathbbox /charht exch def pop pop pop grestore "
		"charht 2 div sub moveto show} bind def\n"
		"/l /lineto load def\n"
		"%%EndProlog\n"
		"gsave\n"
		"/Helvetica findfont 10 scalefont setfont\n"
		"1 setlinecap\n"
		"1 setlinejoin\n"
		".5 setlinewidth\n");
}

/* close the plotting output */
void
closepl(void)
{
	print("showpage\ngrestore\n");
}


/* plot a point at _x,_y, which becomes current */
void
point(int _x, int _y)
{
	print("%.1f %.1f m\n",xs(_x),ys(_y));
}

/* place text, first letter at current point, which does not change */
void
text(char *_s)
{
	print("(%s) %.1f %.1f t\n",_s,_ox,_oy);
}

/* draw line from current point to _x,_y, which becomes current */
void
lineto(int _x, int _y)
{
	print("%.1f %.1f l\n", xs(_x),ys(_y));
}

void 
extern void curveto(int px, int py, int qx, int qy, int x, int y)
{
	print("%.1f %.1f %.1f %.1f %.1f %.1f c\n", xs(px), ys(py), xs(qx), ys(qy), xs(x),ys(y));
}

/* _x,_y becomes current point */
void
moveto(int _x, int _y)
{
	print("%.1f %.1f m\n", xs(_x), ys(_y));
}

void
stroke(void)
{
	print("s\n");
}

void
fill(void)
{
	print("f\n");
}

void
newpath(void)
{
	print("np\n");
}


void
closepath(void)
{
	print("cp\n");
}


struct dashpat {
	char *name;
	char *pat;
	double width;
} pattern[] = {
	SOLID, "[] 0", .5,		/* pattern, minimum acceptable width */
	DOTTED, "[3 6] 0", 0,
	DOTDASH, "[18 6 3 6] 0", .5,
};

/* specify style for drawing lines: "dotted", "solid", "dotdash" */
void
pen(char *s)
{
	int i;
	char *t;
	double w = width;
	for(i=0; i<sizeof(pattern)/sizeof(*pattern); i++)
		if(strcmp(s,pattern[i].name) == 0)
			break;
	if(i >= sizeof(pattern)/sizeof(*pattern)) {
		while(t = strchr(s, ','))
			*t = ' ';
	} else if(width <= pattern[i].width)
		w = pattern[i].width;
	print("s\n%.2f setlinewidth\n%s setdash\n",
		w, pattern[i].pat);
}

static struct {
	char *name;
	char *hsbcode;
} colors[] = {
	BLACK, "0 0 0",
	RED, "0 1 1",
	GREEN, ".33 1 1",
	BLUE, ".67 1 1"
};


void
weight(double w)
{
	width = w;
}

void
comment(char *s, char *t)
{
	print("%%%s %s\n",s,t);
}
